<?php
/*
* 2007-2016 PrestaShop
*
* NOTICE OF LICENSE
*
* This source file is subject to the Academic Free License (AFL 3.0)
* that is bundled with this package in the file LICENSE.txt.
* It is also available through the world-wide-web at this URL:
* http://opensource.org/licenses/afl-3.0.php
* If you did not receive a copy of the license and are unable to
* obtain it through the world-wide-web, please send an email
* to license@prestashop.com so we can send you a copy immediately.
*
* DISCLAIMER
*
* Do not edit or add to this file if you wish to upgrade PrestaShop to newer
* versions in the future. If you wish to customize PrestaShop for your
* needs please refer to http://www.prestashop.com for more information.
*
*  @author PrestaShop SA <contact@prestashop.com>
*  @copyright  2007-2016 PrestaShop SA
*  @version  Release: $Revision$
*  @license    http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
*  International Registered Trademark & Property of PrestaShop SA
*/

if (!defined('_PS_VERSION_')) {
    exit;
}

use PrestaShop\PrestaShop\Core\Module\WidgetInterface;
use PrestaShop\PrestaShop\Adapter\Category\CategoryProductSearchProvider;
use PrestaShop\PrestaShop\Adapter\Image\ImageRetriever;
use PrestaShop\PrestaShop\Adapter\Product\PriceFormatter;
use PrestaShop\PrestaShop\Core\Product\ProductListingPresenter;
use PrestaShop\PrestaShop\Adapter\Product\ProductColorsRetriever;
use PrestaShop\PrestaShop\Core\Product\Search\ProductSearchContext;
use PrestaShop\PrestaShop\Core\Product\Search\ProductSearchQuery;
use PrestaShop\PrestaShop\Core\Product\Search\SortOrder;

class PeleTagproducts extends Module implements WidgetInterface
{
    protected $html;
    protected $templateFile;

    public function __construct()
    {
        $this->name = 'peletagproducts';
        $this->author = 'Pelechano.es';
        $this->version = '1.0.0';

        $this->bootstrap = true;
        parent::__construct();

        $this->displayName = $this->l('Products with the same tag');
        $this->description = $this->l('Adds a block on the product page that displays products from the same tag.');
        $this->ps_versions_compliancy = array('min' => '1.7.5.0', 'max' => _PS_VERSION_);

        $this->templateFile = 'module:peletagproducts/views/templates/hook/peletagproducts.tpl';

        if(Configuration::get('PELETAGPRODUCTS_THEME')==1){
            $this->templateFile = 'module:peletagproducts/views/templates/hook/peletagproducts_warehouse.tpl';
        }

    }

    public function install()
    {
        return (parent::install()
            && Configuration::updateValue('PELETAGPRODUCTS_DISPLAY_PRICE', 1)
            && Configuration::updateValue('PELETAGPRODUCTS_DISPLAY_PRODUCTS', 16)
            && Configuration::updateValue('PELETAGPRODUCTS_THEME', 0)
            && $this->registerHook('displayFooterProduct')
            && $this->registerHook('actionProductAdd')
            && $this->registerHook('actionProductUpdate')
            && $this->registerHook('actionProductDelete')
        );
    }

    public function uninstall()
    {
        if (!parent::uninstall() ||
            !Configuration::deleteByName('PELETAGPRODUCTS_DISPLAY_PRICE') || !Configuration::deleteByName('PELETAGPRODUCTS_THEME') ||
            !Configuration::deleteByName('PELETAGPRODUCTS_DISPLAY_PRODUCTS')) {
            return false;
        }
        return true;
    }

    public function getContent()
    {
        $this->html = '';

        if (Tools::isSubmit('submitCross')) {
            $isValidDisplayPrice = Tools::getValue('PELETAGPRODUCTS_DISPLAY_PRICE') === '0' || Tools::getValue('PELETAGPRODUCTS_DISPLAY_PRICE') === '1';
            if (false === $isValidDisplayPrice) {
                $this->html .= $this->displayError($this->l('Invalid value for display price.'));
            }

            if ($isValidDisplayPrice) {
                Configuration::updateValue('PELETAGPRODUCTS_DISPLAY_PRICE', Tools::getValue('PELETAGPRODUCTS_DISPLAY_PRICE'));
                Configuration::updateValue('PELETAGPRODUCTS_DISPLAY_PRODUCTS', (int) Tools::getValue('PELETAGPRODUCTS_DISPLAY_PRODUCTS'));
                Configuration::updateValue('PELETAGPRODUCTS_THEME', (int) Tools::getValue('PELETAGPRODUCTS_THEME'));

                $this->_clearCache($this->templateFile);
                $this->html .= $this->displayConfirmation($this->l('The settings have been updated.'));
            }
        }

        $this->html .= $this->renderForm();

        return $this->html;
    }

    public function hookAddProduct($params)
    {
        return $this->clearCache($params);
    }

    public function hookUpdateProduct($params)
    {
        return $this->clearCache($params);
    }

    public function hookDeleteProduct($params)
    {
        return $this->clearCache($params);
    }

    private function clearCache($params)
    {
        $params = $this->getInformationFromConfiguration($params);

        if ($params) {
            $this->_clearCache($this->templateFile, $params['cache_id']);
        } else {
            $this->_clearCache($this->templateFile);
        }

        return;
    }

    public function renderForm()
    {
        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('Settings'),
                    'icon' => 'icon-cogs',
                ),
                'input' => array(
                    array(
                        'type' => 'switch',
                        'label' => $this->l('Display products\' prices'),
                        'name' => 'PELETAGPRODUCTS_DISPLAY_PRICE',
                        'values' => array(
                            array(
                                'id' => 'active_on',
                                'value' => 1,
                                'label' => $this->l('Enabled'),
                            ),
                            array(
                                'id' => 'active_off',
                                'value' => 0,
                                'label' => $this->l('Disabled'),
                            )
                        ),
                    ),
                    array(
                        'type' => 'text',
                        'label' => $this->l('Number of product to display'),
                        'name' => 'PELETAGPRODUCTS_DISPLAY_PRODUCTS',
                        'class' => 'fixed-width-xs',
                    ),
                    array(
                        'label' => $this->l('Theme'),
                        'type' => 'radio',
                        'name' => 'PELETAGPRODUCTS_THEME',
                        'values' => array(
                            array(
                                'id' => 'PELETAGPRODUCTS_THEME_0',
                                'value' => 0,
                                'label' => $this->l('Classic'),
                            ),
                            array(
                                'id' => 'PELETAGPRODUCTS_THEME_1',
                                'value' => 1,
                                'label' => $this->l('Warehouse '),
                            ),

                        ),
                     )
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                ),
            ),
        );

        $lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));

        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get(
            'PS_BO_ALLOW_EMPLOYEE_FORM_LANG'
        ) : 0;
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitCross';
        $helper->currentIndex = $this->context->link->getAdminLink(
                'AdminModules',
                false
            ).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id
        );

        return $helper->generateForm(array($fields_form));
    }

    public function getConfigFieldsValues()
    {
        return array(
            'PELETAGPRODUCTS_DISPLAY_PRICE' => Configuration::get('PELETAGPRODUCTS_DISPLAY_PRICE'),
            'PELETAGPRODUCTS_DISPLAY_PRODUCTS' => Configuration::get('PELETAGPRODUCTS_DISPLAY_PRODUCTS'),
            'PELETAGPRODUCTS_THEME' => Configuration::get('PELETAGPRODUCTS_THEME'),
        );
    }

    public function getWidgetVariables($hookName = null, array $configuration = array())
    {
        $params = $this->getInformationFromConfiguration($configuration);

        if ($params) {

            $products = $this->getTagProducts($params['id_product']);

            if (!empty($products)) {
                return array(
                    'products' => $products,
                );
            }

        }

        return false;
    }

    public function renderWidget($hookName = null, array $configuration = array())
    {
        $params = $this->getInformationFromConfiguration($configuration);

        if ($params) {
            if ((int)Configuration::get('PELETAGPRODUCTS_DISPLAY_PRODUCTS') > 0) {

                // Need variables only if this template isn't cached
                if (!$this->isCached($this->templateFile, $params['cache_id'])) {
                    if (!empty($params['id_product'])) {
                        $product = new Product($params['id_product']);
                    }

                    if (empty($product) || !Validate::isLoadedObject($product) || !$product->active) {
                        return false;
                    }

                    $variables = $this->getWidgetVariables($hookName, $configuration);

                    if (empty($variables)) {
                        return false;
                    }

                    $this->smarty->assign($variables);
                }

                return $this->fetch(
                    $this->templateFile,
                    $params['cache_id']
                );
            }
        }

        return false;
    }

    private function getTagProducts($idProduct)
    {

        $context=$this->context;

        $idLang = $context->language->id;
        $active=true;
        $front=true;

        $taglist = self::getProductTagsIds($idProduct,$idLang);

        if(!$taglist){
            return false;
        }


        $tags_id_str =join(',',$taglist);

        $showPrice = (bool) Configuration::get('PELETAGPRODUCTS_DISPLAY_PRICE');

        $nbDaysNewProduct = Configuration::get('PS_NB_DAYS_NEW_PRODUCT');
        if (!Validate::isUnsignedInt($nbDaysNewProduct)) {
            $nbDaysNewProduct = 20;
        }



        $sql = 'SELECT p.*, product_shop.*, stock.out_of_stock, IFNULL(stock.quantity, 0) AS quantity' . (Combination::isFeatureActive() ? ', IFNULL(product_attribute_shop.id_product_attribute, 0) AS id_product_attribute,
					product_attribute_shop.minimal_quantity AS product_attribute_minimal_quantity' : '') . ', pl.`description`, pl.`description_short`, pl.`available_now`,
					pl.`available_later`, pl.`link_rewrite`, pl.`meta_description`, pl.`meta_keywords`, pl.`meta_title`, pl.`name`, image_shop.`id_image` id_image,
					il.`legend` as legend, m.`name` AS manufacturer_name, cl.`name` AS category_default,
					DATEDIFF(product_shop.`date_add`, DATE_SUB("' . date('Y-m-d') . ' 00:00:00",
					INTERVAL ' . (int) $nbDaysNewProduct . ' DAY)) > 0 AS new, product_shop.price AS orderprice
				FROM `' . _DB_PREFIX_ . 'product_tag` pt
				LEFT JOIN `' . _DB_PREFIX_ . 'product` p
					ON p.`id_product` = pt.`id_product`
				' . Shop::addSqlAssociation('product', 'p') .
            (Combination::isFeatureActive() ? ' LEFT JOIN `' . _DB_PREFIX_ . 'product_attribute_shop` product_attribute_shop
				ON (p.`id_product` = product_attribute_shop.`id_product` AND product_attribute_shop.`default_on` = 1 AND product_attribute_shop.id_shop=' . (int) $context->shop->id . ')' : '') . '
				' . Product::sqlStock('p', 0) . '
				LEFT JOIN `' . _DB_PREFIX_ . 'category_lang` cl
					ON (product_shop.`id_category_default` = cl.`id_category`
					AND cl.`id_lang` = ' . (int) $idLang . Shop::addSqlRestrictionOnLang('cl') . ')
				LEFT JOIN `' . _DB_PREFIX_ . 'product_lang` pl
					ON (p.`id_product` = pl.`id_product`
					AND pl.`id_lang` = ' . (int) $idLang . Shop::addSqlRestrictionOnLang('pl') . ')
				LEFT JOIN `' . _DB_PREFIX_ . 'image_shop` image_shop
					ON (image_shop.`id_product` = p.`id_product` AND image_shop.cover=1 AND image_shop.id_shop=' . (int) $context->shop->id . ')
				LEFT JOIN `' . _DB_PREFIX_ . 'image_lang` il
					ON (image_shop.`id_image` = il.`id_image`
					AND il.`id_lang` = ' . (int) $idLang . ')
				LEFT JOIN `' . _DB_PREFIX_ . 'manufacturer` m
					ON m.`id_manufacturer` = p.`id_manufacturer`
				WHERE product_shop.`id_shop` = ' . (int) $context->shop->id . '
					AND pt.`id_tag` IN (' . (int) $tags_id_str.') '
            . ($active ? ' AND product_shop.`active` = 1' : '')
            . ($front ? ' AND product_shop.`visibility` IN ("both", "catalog")' : '');


        $nProducts = (int) Configuration::get('PELETAGPRODUCTS_DISPLAY_PRODUCTS') + 1; // +1 If current product is found
        $sql .= ' ORDER BY p.id_product DESC LIMIT ' . (int) $nProducts;



        $products = DB::getInstance()->executeS($sql);



        $productsForTemplate = array();



        $assembler = new ProductAssembler($this->context);
        $presenterFactory = new ProductPresenterFactory($this->context);
        $presentationSettings = $presenterFactory->getPresentationSettings();
        $presenter = new ProductListingPresenter(
            new ImageRetriever(
                $this->context->link
            ),
            $this->context->link,
            new PriceFormatter(),
            new ProductColorsRetriever(),
            $this->context->getTranslator()
        );
        $presentationSettings->showPrices = $showPrice;


        foreach ($products as $rawProduct) {
            // Not duplicate current product
            if ($rawProduct['id_product'] != $idProduct && count($productsForTemplate) < (int) Configuration::get('PELETAGPRODUCTS_DISPLAY_PRODUCTS')) {
                $productsForTemplate[] = $presenter->present(
                    $presentationSettings,
                    $assembler->assembleProduct($rawProduct),
                    $this->context->language
                );
            }
        }

        return $productsForTemplate;
    }

    private function getInformationFromConfiguration($configuration)
    {
        if (empty($configuration['product'])) {
            return false;
        }

        $product = $configuration['product'];
        if ($product instanceof Product) {
            $product = (array) $product;
            $product['id_product'] = $product['id'];
        }

        $id_product = $product['id_product'];
        $id_category = (isset($configuration['category']->id) ? (int) $configuration['category']->id : (int) $product['id_category_default']);

        if (!empty($id_product) && !empty($id_category)) {

            $cache_id = 'peletagproducts|'.$id_product.'|'.$id_category;

            return array(
                'id_product' => $id_product,
                'id_category' => $id_category,
                'cache_id' => $this->getCacheId($cache_id),
            );
        }

        return false;
    }

    private static function getProductTagsIds($idProduct,$id_lang)
    {
        if (!$tmp = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS('
        SELECT DISTINCT(pt.`id_tag`)
        FROM ' . _DB_PREFIX_ . 'product_tag pt
        WHERE pt.`id_product`=' . (int) $idProduct.' AND pt.id_lang='. (int)$id_lang)) {
            return false;
        }

        $result = [];
        foreach ($tmp as $tag) {
            $result[]= $tag['id_tag'];
        }

        return $result;
    }


}
